@@ -1,39 +1,40 @@
+import warnings
 from functools import partial
 from itertools import count
-import warnings
-from typing import Any, Dict, Sequence, Literal, Optional, Tuple, Type, Union
+from typing import Any, Dict, Literal, Optional, Sequence, Tuple, Type, Union
+
 import casadi as cs
 import numpy as npy
-from csnlp.solutions import Solution, subsevalf, DMStruct
+
 from csnlp.debug import NlpDebug
-from csnlp.util.funcs import cached_property, cache_clearer, np_random
-from csnlp.util.data import struct_symSX, dict2struct
+from csnlp.solutions import DMStruct, Solution, subsevalf
+from csnlp.util.data import dict2struct, struct_symSX
+from csnlp.util.funcs import cache_clearer, cached_property, np_random
 
-
-'''This tuple dictates the order for operations related to dual variables.'''
-_DUAL_VARIABLES_ORDER = ('g', 'h', 'h_lbx', 'h_ubx')
+"""This tuple dictates the order for operations related to dual variables."""
+_DUAL_VARIABLES_ORDER = ("g", "h", "h_lbx", "h_ubx")
 
 
 class Nlp:
-    '''
+    """
     The generic NLP class is a controller that solves a (possibly, nonlinear)
     optimization problem to yield a (possibly, sub-) optimal solution.
 
     This is a generic class in the sense that it does not solve a particular
     problem, but only offers the generic methods to build one (e.g., variables,
     constraints, objective, solver).
-    '''
+    """
 
     __ids = count(0)
 
     def __init__(
         self,
-        sym_type: Literal['SX', 'MX'] = 'SX',
+        sym_type: Literal["SX", "MX"] = "SX",
         remove_redundant_x_bounds: bool = True,
         name: Optional[str] = None,
-        seed: Optional[int] = None
+        seed: Optional[int] = None,
     ) -> None:
-        '''Creates an NLP instance with a given name.
+        """Creates an NLP instance with a given name.
 
         Parameters
         ----------
@@ -48,10 +49,9 @@ class Nlp:
             Name of the NLP scheme. If `None`, it is automatically assigned.
         seed : int, optional
             Random number generator seed.
-        '''
-
+        """
         self.id = next(self.__ids)
-        self.name = f'NLP{self.id}' if name is None else name
+        self.name = f"{self.__class__.__name__}{self.id}" if name is None else name
         self._csXX: Union[Type[cs.SX], Type[cs.MX]] = getattr(cs, sym_type)
 
         self._vars: Dict[str, Union[cs.SX, cs.MX]] = {}
@@ -77,189 +77,189 @@ class Nlp:
         self._remove_redundant_x_bounds = remove_redundant_x_bounds
 
     @property
-    def unwrapped(self) -> 'Nlp':
-        '''Returns the base non-wrapped NLP.'''
+    def unwrapped(self) -> "Nlp":
+        """Returns the base non-wrapped NLP."""
         return self
 
     @property
     def np_random(self) -> npy.random.Generator:
-        '''Returns the nlp's random engine that, if not set, will be
-        initialised with the nlp's seed.'''
+        """Returns the nlp's random engine that, if not set, will be
+        initialised with the nlp's seed."""
         if self._np_random is None:
             self._np_random, _ = np_random(self._seed)
         return self._np_random
 
     @property
     def sym_type(self) -> Union[Type[cs.SX], Type[cs.MX]]:
-        '''Gets the CasADi symbolic type used in this NLP scheme.'''
+        """Gets the CasADi symbolic type used in this NLP scheme."""
         return self._csXX
 
     @property
     def f(self) -> Union[None, cs.SX, cs.MX]:
-        '''Gets the objective of the NLP scheme, which is `None` if not set
-        previously set via the `minimize` method.'''
+        """Gets the objective of the NLP scheme, which is `None` if not set
+        previously set via the `minimize` method."""
         return self._f
 
     @property
     def p(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the parameters of the NLP scheme.'''
+        """Gets the parameters of the NLP scheme."""
         return self._p
 
     @property
     def x(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the primary variables of the NLP scheme in vector form.'''
+        """Gets the primary variables of the NLP scheme in vector form."""
         return self._x
 
     @property
     def lbx(self) -> npy.ndarray:
-        '''Gets the lower bound constraints of primary variables of the NLP
-        scheme in vector form.'''
+        """Gets the lower bound constraints of primary variables of the NLP
+        scheme in vector form."""
         return self._lbx
 
     @property
     def ubx(self) -> npy.ndarray:
-        '''Gets the upper bound constraints of primary variables of the NLP
-        scheme in vector form.'''
+        """Gets the upper bound constraints of primary variables of the NLP
+        scheme in vector form."""
         return self._ubx
 
     @property
     def lam_lbx(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the dual variables of the primary variables lower bound
-        constraints of the NLP scheme in vector form.'''
+        """Gets the dual variables of the primary variables lower bound
+        constraints of the NLP scheme in vector form."""
         return self._lam_lbx
 
     @property
     def lam_ubx(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the dual variables of the primary variables upper bound
-        constraints of the NLP scheme in vector form.'''
+        """Gets the dual variables of the primary variables upper bound
+        constraints of the NLP scheme in vector form."""
         return self._lam_ubx
 
     @property
     def g(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the equality constraint expressions of the NLP scheme in vector
-        form.'''
+        """Gets the equality constraint expressions of the NLP scheme in vector
+        form."""
         return self._g
 
     @property
     def h(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the inequality constraint expressions of the NLP scheme in
-        vector form.'''
+        """Gets the inequality constraint expressions of the NLP scheme in
+        vector form."""
         return self._h
 
     @property
     def lam_g(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the dual variables of the equality constraints of the NLP
-        scheme in vector form.'''
+        """Gets the dual variables of the equality constraints of the NLP
+        scheme in vector form."""
         return self._lam_g
 
     @property
     def lam_h(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the dual variables of the inequality constraints of the NLP
-        scheme in vector form.'''
+        """Gets the dual variables of the inequality constraints of the NLP
+        scheme in vector form."""
         return self._lam_h
 
     @property
     def solver(self) -> Optional[cs.Function]:
-        '''Gets the NLP optimization solver. Can be `None`, if the solver is
-        not set with method `init_solver`.'''
+        """Gets the NLP optimization solver. Can be `None`, if the solver is
+        not set with method `init_solver`."""
         return self._solver
 
     @property
     def solver_opts(self) -> Dict[str, Any]:
-        '''Gets the NLP optimization solver options. The dict is empty, if the
-        solver options are not set with method `init_solver`.'''
+        """Gets the NLP optimization solver options. The dict is empty, if the
+        solver options are not set with method `init_solver`."""
         return self._solver_opts
 
     @property
     def failures(self) -> int:
-        '''Gets the cumulative number of failures of the NLP solver.'''
+        """Gets the cumulative number of failures of the NLP solver."""
         return self._failures
 
     @property
     def debug(self) -> NlpDebug:
-        '''Gets debug information on the NLP scheme.'''
+        """Gets debug information on the NLP scheme."""
         return self._debug
 
     @property
     def nx(self) -> int:
-        '''Number of variables in the NLP scheme.'''
+        """Number of variables in the NLP scheme."""
         return self._x.shape[0]
 
     @property
     def np(self) -> int:
-        '''Number of parameters in the NLP scheme.'''
+        """Number of parameters in the NLP scheme."""
         return self._p.shape[0]
 
     @property
     def ng(self) -> int:
-        '''Number of equality constraints in the NLP scheme.'''
+        """Number of equality constraints in the NLP scheme."""
         return self._g.shape[0]
 
     @property
     def nh(self) -> int:
-        '''Number of inequality constraints in the NLP scheme.'''
+        """Number of inequality constraints in the NLP scheme."""
         return self._h.shape[0]
 
     @cached_property
     def parameters(self) -> Union[struct_symSX, Dict[str, cs.MX]]:
-        '''Gets the parameters of the NLP scheme.'''
+        """Gets the parameters of the NLP scheme."""
         return dict2struct(self._pars)
 
     @cached_property
     def variables(self) -> Union[struct_symSX, Dict[str, cs.MX]]:
-        '''Gets the primal variables of the NLP scheme.'''
+        """Gets the primal variables of the NLP scheme."""
         return dict2struct(self._vars)
 
     @cached_property
     def dual_variables(self) -> Union[struct_symSX, Dict[str, cs.MX]]:
-        '''Gets the dual variables of the NLP scheme.'''
+        """Gets the dual variables of the NLP scheme."""
         return dict2struct(self._dual_vars)
 
     @cached_property
     def constraints(self) -> Union[struct_symSX, Dict[str, cs.MX]]:
-        '''Gets the constraints of the NLP scheme.'''
-        return dict2struct(self._cons, entry_type='expr')
+        """Gets the constraints of the NLP scheme."""
+        return dict2struct(self._cons, entry_type="expr")
 
     @cached_property
     def lam(self) -> Union[cs.SX, cs.MX]:
-        '''Gets the dual variables of the NLP scheme in vector form.
+        """Gets the dual variables of the NLP scheme in vector form.
 
         Note: The order of the dual variables can be adjusted via
-        `_DUAL_VARIABLES_ORDER`.'''
+        `_DUAL_VARIABLES_ORDER`."""
         items = {
-            'g': self._lam_g,
-            'h': self._lam_h,
-            'h_lbx': self.h_lbx[1],
-            'h_ubx': self.h_ubx[1]
+            "g": self._lam_g,
+            "h": self._lam_h,
+            "h_lbx": self.h_lbx[1],
+            "h_ubx": self.h_ubx[1],
         }
         dual = cs.vertcat(*(items.pop(v) for v in _DUAL_VARIABLES_ORDER))
-        assert not items, 'Internal error. _DUAL_VARIABLES_ORDER modified.'
+        assert not items, "Internal error. _DUAL_VARIABLES_ORDER modified."
         return dual
 
     @cached_property
     def lam_all(self) -> Union[cs.SX, cs.MX]:
-        '''Gets all the dual variables of the NLP scheme in vector form,
+        """Gets all the dual variables of the NLP scheme in vector form,
         irrespective of redundant `lbx` and `ubx` multipliers. If
         `remove_redundant_x_bounds`, then this property is equivalent to
         the `lam` property.
 
         Note: The order of the dual variables can be adjusted via
-        `_DUAL_VARIABLES_ORDER`.'''
+        `_DUAL_VARIABLES_ORDER`."""
         items = {
-            'g': self._lam_g,
-            'h': self._lam_h,
-            'h_lbx': self._lam_lbx,
-            'h_ubx': self._lam_ubx
+            "g": self._lam_g,
+            "h": self._lam_h,
+            "h_lbx": self._lam_lbx,
+            "h_ubx": self._lam_ubx,
         }
         dual = cs.vertcat(*(items.pop(v) for v in _DUAL_VARIABLES_ORDER))
-        assert not items, 'Internal error. _DUAL_VARIABLES_ORDER modified.'
+        assert not items, "Internal error. _DUAL_VARIABLES_ORDER modified."
         return dual
 
     @cached_property
     def h_lbx(self) -> Union[Tuple[cs.SX, cs.SX], Tuple[cs.MX, cs.MX]]:
-        '''Gets the inequalities due to `lbx` and their multipliers. If
+        """Gets the inequalities due to `lbx` and their multipliers. If
         `simplify_x_bounds=True`, it removes redundant entries, i.e., where
-        `lbx == -inf`; otherwise, returns all lower bound constraints.'''
+        `lbx == -inf`; otherwise, returns all lower bound constraints."""
         if not self._remove_redundant_x_bounds:
             return self._lbx[:, None] - self._x, self._lam_lbx
         idx = npy.where(self._lbx != -npy.inf)[0]
@@ -269,9 +269,9 @@ class Nlp:
 
     @cached_property
     def h_ubx(self) -> Union[Tuple[cs.SX, cs.SX], Tuple[cs.MX, cs.MX]]:
-        '''Gets the inequalities due to `ubx` and their multipliers. If
+        """Gets the inequalities due to `ubx` and their multipliers. If
         `simplify_x_bounds=True`, it removes redundant entries, i.e., where
-        `ubx == +inf`; otherwise, returns all upper bound constraints.'''
+        `ubx == +inf`; otherwise, returns all upper bound constraints."""
         if not self._remove_redundant_x_bounds:
             return self._x - self._ubx[:, None], self._lam_ubx
         idx = npy.where(self._ubx != npy.inf)[0]
@@ -280,7 +280,7 @@ class Nlp:
         return self._x[idx] - self._ubx[idx, None], self._lam_ubx[idx]
 
     def primal_dual_vars(self, all: bool = False) -> Union[cs.SX, cs.MX]:
-        '''Gets the collection of primal-dual variables (usually, denoted as
+        """Gets the collection of primal-dual variables (usually, denoted as
         `y`)
         ```
                     y = [x^T, lam^T]^T
@@ -303,16 +303,14 @@ class Nlp:
         ----
         The order of the dual variables can be adjusted via
         `_DUAL_VARIABLES_ORDER`.
-        '''
+        """
         return cs.vertcat(self._x, self.lam_all if all else self.lam)
 
     @cache_clearer(parameters)
     def parameter(
-        self,
-        name: str,
-        shape: Tuple[int, int] = (1, 1)
+        self, name: str, shape: Tuple[int, int] = (1, 1)
     ) -> Union[cs.SX, cs.MX]:
-        '''Adds a parameter to the NLP scheme.
+        """Adds a parameter to the NLP scheme.
 
         Parameters
         ----------
@@ -330,13 +328,13 @@ class Nlp:
         ------
         ValueError
             Raises if there is already another parameter with the same name.
-        '''
+        """
         if name in self._pars:
-            raise ValueError(f'Parameter name \'{name}\' already exists.')
+            raise ValueError(f"Parameter name '{name}' already exists.")
         par = self._csXX.sym(name, *shape)
         self._pars[name] = par
         self._p = cs.vertcat(self._p, cs.vec(par))
-        self._debug.register('p', name, shape)
+        self._debug.register("p", name, shape)
 
         if self._solver is not None:
             self.init_solver(self._solver_opts)  # resets solver
@@ -348,9 +346,9 @@ class Nlp:
         name: str,
         shape: Tuple[int, int] = (1, 1),
         lb: Union[npy.ndarray, cs.DM] = -npy.inf,
-        ub: Union[npy.ndarray, cs.DM] = +npy.inf
+        ub: Union[npy.ndarray, cs.DM] = +npy.inf,
     ) -> Union[Tuple[cs.SX, cs.SX, cs.SX], Tuple[cs.MX, cs.MX, cs.MX]]:
-        '''
+        """
         Adds a variable to the NLP problem.
 
         Parameters
@@ -379,25 +377,25 @@ class Nlp:
             Raises if there is already another variable with the same name; or
             if any element of the lower bound is larger than the corresponding
             lower bound element.
-        '''
+        """
         if name in self._vars:
-            raise ValueError(f'Variable name \'{name}\' already exists.')
+            raise ValueError(f"Variable name '{name}' already exists.")
         lb, ub = npy.broadcast_to(lb, shape), npy.broadcast_to(ub, shape)
         if npy.all(lb > ub):
-            raise ValueError('Improper variable bounds.')
+            raise ValueError("Improper variable bounds.")
 
         var = self._csXX.sym(name, *shape)
         self._vars[name] = var
         self._x = cs.vertcat(self._x, cs.vec(var))
-        self._lbx = npy.concatenate((self._lbx, lb.flatten('F')))
-        self._ubx = npy.concatenate((self._ubx, ub.flatten('F')))
-        self._debug.register('x', name, shape)
+        self._lbx = npy.concatenate((self._lbx, lb.flatten("F")))
+        self._ubx = npy.concatenate((self._ubx, ub.flatten("F")))
+        self._debug.register("x", name, shape)
 
-        name_lam = f'lam_lb_{name}'
+        name_lam = f"lam_lb_{name}"
         lam_lb = self._csXX.sym(name_lam, *shape)
         self._dual_vars[name_lam] = lam_lb
         self._lam_lbx = cs.vertcat(self._lam_lbx, cs.vec(lam_lb))
-        name_lam = f'lam_ub_{name}'
+        name_lam = f"lam_ub_{name}"
         lam_ub = self._csXX.sym(name_lam, *shape)
         self._dual_vars[name_lam] = lam_ub
         self._lam_ubx = cs.vertcat(self._lam_ubx, cs.vec(lam_ub))
@@ -411,17 +409,17 @@ class Nlp:
         self,
         name: str,
         lhs: Union[npy.ndarray, cs.DM, cs.SX, cs.MX],
-        op: Literal['==', '>=', '<='],
+        op: Literal["==", ">=", "<="],
         rhs: Union[npy.ndarray, cs.DM, cs.SX, cs.MX],
         soft: bool = False,
-        simplify: bool = True
+        simplify: bool = True,
     ) -> Union[
         Tuple[cs.SX, cs.SX],
         Tuple[cs.MX, cs.MX],
         Tuple[cs.SX, cs.SX, cs.SX],
         Tuple[cs.MX, cs.MX, cs.MX],
     ]:
-        '''Adds a constraint to the NLP problem, e.g., `lhs <= rhs`.
+        """Adds a constraint to the NLP problem, e.g., `lhs <= rhs`.
 
         Parameters
         ----------
@@ -462,41 +460,43 @@ class Nlp:
             Raises if a soft equality constraint is requested.
         TypeError
             Raises if the constraint is not symbolic.
-        '''
+        """
         if name in self._cons:
-            raise ValueError(f'Constraint name \'{name}\' already exists.')
+            raise ValueError(f"Constraint name '{name}' already exists.")
         expr: Union[cs.SX, cs.MX] = lhs - rhs
         if not isinstance(expr, (cs.SX, cs.MX)):
-            raise TypeError('Constraint must be symbolic.')
+            raise TypeError("Constraint must be symbolic.")
         if simplify:
             expr = cs.simplify(expr)
 
         shape = expr.shape
-        if op == '==':
+        if op == "==":
             is_eq = True
             lb = npy.zeros(npy.prod(shape))
             if soft:
                 raise NotImplementedError(
-                    'Soft equality constraints are not currently supported.')
-        elif op == '<=':
+                    "Soft equality constraints are not currently supported."
+                )
+        elif op == "<=":
             is_eq = False
             lb = npy.full(npy.prod(shape), -npy.inf)
-        elif op == '>=':
+        elif op == ">=":
             is_eq = False
             expr = -expr
             lb = npy.full(npy.prod(shape), -npy.inf)
         else:
-            raise ValueError(f'Unrecognized operator {op}.')
+            raise ValueError(f"Unrecognized operator {op}.")
 
         if soft:
-            slack = self.variable(f'slack_{name}', expr.shape, lb=0)[0]
+            slack = self.variable(f"slack_{name}", expr.shape, lb=0)[0]
             expr -= slack
 
         self._cons[name] = expr
-        group, con, lam = \
-            ('_g', '_lbg', '_lam_g') if is_eq else ('_h', '_lbh', '_lam_h')
+        group, con, lam = (
+            ("_g", "_lbg", "_lam_g") if is_eq else ("_h", "_lbh", "_lam_h")
+        )
         self._debug.register(group[1:], name, shape)
-        name_lam = f'{lam[1:]}_{name}'
+        name_lam = f"{lam[1:]}_{name}"
         lam_c = self._csXX.sym(name_lam, *shape)
         self._dual_vars[name_lam] = lam_c
 
@@ -509,7 +509,7 @@ class Nlp:
         return (expr, lam_c, slack) if soft else (expr, lam_c)
 
     def minimize(self, objective: Union[cs.SX, cs.MX]) -> None:
-        '''Sets the objective function to be minimized.
+        """Sets the objective function to be minimized.
 
         Parameters
         ----------
@@ -521,15 +521,15 @@ class Nlp:
         ------
         ValueError
             Raises if the objective is not scalar.
-        '''
+        """
         if objective.shape != (1, 1):
-            raise ValueError('Objective must be scalar.')
+            raise ValueError("Objective must be scalar.")
         self._f = objective
         if self._solver is not None:
             self.init_solver(self._solver_opts)  # resets solver
 
     def init_solver(self, opts: Optional[Dict[str, Any]] = None) -> None:
-        '''Initializes the IPOPT solver for this NLP with the given options.
+        """Initializes the IPOPT solver for this NLP with the given options.
 
         Parameters
         ----------
@@ -540,23 +540,23 @@ class Nlp:
         ------
         RuntimeError
             Raises if the objective has not yet been specified with `minimize`.
-        '''
+        """
         if self._f is None:
-            raise RuntimeError('NLP objective not set.')
+            raise RuntimeError("NLP objective not set.")
         if opts is None:
             opts = {}
         opts = opts.copy()
         con = cs.vertcat(self._g, self._h)
-        nlp = {'x': self._x, 'p': self._p, 'g': con, 'f': self._f}
-        self._solver = cs.nlpsol(f'nlpsol_{self.name}', 'ipopt', nlp, opts)
+        nlp = {"x": self._x, "p": self._p, "g": con, "f": self._f}
+        self._solver = cs.nlpsol(f"nlpsol_{self.name}", "ipopt", nlp, opts)
         self._solver_opts = opts
 
     def solve(
         self,
         pars: Union[None, DMStruct, Dict[str, npy.ndarray]] = None,
-        vals0: Union[None, DMStruct, Dict[str, npy.ndarray]] = None
+        vals0: Union[None, DMStruct, Dict[str, npy.ndarray]] = None,
     ) -> Solution:
-        '''Solves the NLP optimization problem.
+        """Solves the NLP optimization problem.
 
         Parameters
         ----------
@@ -579,51 +579,55 @@ class Nlp:
         RuntimeError
             Raises if the solver is un-initialized (see `init_solver`); or if
             not all the parameters are not provided with a numerical value.
-        '''
+        """
         if pars is None:
             pars = {}
         if self._solver is None:
-            raise RuntimeError('Solver uninitialized.')
+            raise RuntimeError("Solver uninitialized.")
         parsdiff = self._pars.keys() - pars.keys()
         if len(parsdiff) != 0:
             raise RuntimeError(
-                'Trying to solve the NLP with unspecified parameters: ' +
-                ', '.join(parsdiff) + '.')
+                "Trying to solve the NLP with unspecified parameters: "
+                + ", ".join(parsdiff)
+                + "."
+            )
 
         p = subsevalf(self._p, self._pars, pars)
         kwargs = {
-            'p': p,
-            'lbx': self._lbx,
-            'ubx': self._ubx,
-            'lbg': npy.concatenate((self._lbg, self._lbh)),
-            'ubg': 0,
+            "p": p,
+            "lbx": self._lbx,
+            "ubx": self._ubx,
+            "lbg": npy.concatenate((self._lbg, self._lbh)),
+            "ubg": 0,
         }
         if vals0 is not None:
-            kwargs['x0'] = subsevalf(self._x, self._vars, vals0)
+            kwargs["x0"] = subsevalf(self._x, self._vars, vals0)
         sol: Dict[str, cs.DM] = self._solver(**kwargs)
 
         # extract lam_x, lam_g and lam_h
-        lam_lbx = -cs.fmin(sol['lam_x'], 0)
-        lam_ubx = cs.fmax(sol['lam_x'], 0)
-        lam_g = sol['lam_g'][:self.ng, :]
-        lam_h = sol['lam_g'][self.ng:, :]
+        lam_lbx = -cs.fmin(sol["lam_x"], 0)
+        lam_ubx = cs.fmax(sol["lam_x"], 0)
+        lam_g = sol["lam_g"][: self.ng, :]
+        lam_h = sol["lam_g"][self.ng :, :]
 
         vars_ = self.variables
         if self._csXX is cs.SX:
-            vals = vars_(sol['x'])
+            vals = vars_(sol["x"])
         else:
-            vals = dict2struct({name: subsevalf(var, self._x, sol['x'])
-                                for name, var in vars_.items()})
-        old = cs.vertcat(self._p, self._x, self._lam_g, self._lam_h,
-                         self._lam_lbx, self._lam_ubx)
-        new = cs.vertcat(p, sol['x'], lam_g, lam_h, lam_lbx, lam_ubx)
+            vals = dict2struct(
+                {name: subsevalf(var, self._x, sol["x"]) for name, var in vars_.items()}
+            )
+        old = cs.vertcat(
+            self._p, self._x, self._lam_g, self._lam_h, self._lam_lbx, self._lam_ubx
+        )
+        new = cs.vertcat(p, sol["x"], lam_g, lam_h, lam_lbx, lam_ubx)
         get_value = partial(subsevalf, old=old, new=new)
         solution = Solution(
-            f=float(sol['f']),
+            f=float(sol["f"]),
             vars=vars_,
             vals=vals,
             stats=self._solver.stats().copy(),
-            _get_value=get_value
+            _get_value=get_value,
         )
         self._failures += int(not solution.success)
         return solution
@@ -633,11 +637,11 @@ class Nlp:
         name: str,
         ins: Union[Sequence[cs.SX], Sequence[cs.MX]],
         outs: Union[Sequence[cs.SX], Sequence[cs.MX]],
-        name_in: Sequence[str] = None,
-        name_out: Sequence[str] = None,
-        opts: Dict[Any, Any] = None
+        name_in: Optional[Sequence[str]] = None,
+        name_out: Optional[Sequence[str]] = None,
+        opts: Optional[Dict[Any, Any]] = None,
     ) -> cs.Function:
-        '''Converts the optimization problem to an MX symbolic function. If the
+        """Converts the optimization problem to an MX symbolic function. If the
         NLP is modelled in SX, the function will still be converted in MX since
         the IPOPT interface cannot expand SX for now.
 
@@ -672,27 +676,35 @@ class Nlp:
         ValueError
             Raises if the input or output expressions have free variables that
             are not provided or cannot be computed by the solver.
-        '''
+        """
         if self._csXX is cs.SX:
-            warnings.warn('The IPOPT interface does not support SX expansion, '
-                          'so the function must be wrapped in MX.',
-                          RuntimeWarning)
+            warnings.warn(
+                "The IPOPT interface does not support SX expansion, "
+                "so the function must be wrapped in MX.",
+                RuntimeWarning,
+            )
         S = self._solver
         if S is None:
-            raise RuntimeError('Solver not yet initialized.')
+            raise RuntimeError("Solver not yet initialized.")
 
         # converts inputs/outputs to/from variables and parameters
         n_outs = len(outs)
-        Fin = cs.Function('Fin', ins, [self._x, self._p])
-        Fout = cs.Function('Fout', [
-            self._p, self._x, self._lam_g, self._lam_h, self._lam_lbx,
-            self._lam_ubx], outs)
+        Fin = cs.Function("Fin", ins, [self._x, self._p])
+        Fout = cs.Function(
+            "Fout",
+            [self._p, self._x, self._lam_g, self._lam_h, self._lam_lbx, self._lam_ubx],
+            outs,
+        )
         if Fin.has_free():
-            raise ValueError('Input expressions do not provide values for: '
-                             f'{", ".join(Fin.get_free())}.')
+            raise ValueError(
+                "Input expressions do not provide values for: "
+                f'{", ".join(Fin.get_free())}.'
+            )
         if Fout.has_free():
-            raise ValueError('Output solver cannot provide values for: '
-                             f'{", ".join(Fout.get_free())}.')
+            raise ValueError(
+                "Output solver cannot provide values for: "
+                f'{", ".join(Fout.get_free())}.'
+            )
 
         # call the solver
         if self._csXX is cs.SX:
@@ -709,14 +721,14 @@ class Nlp:
             lbg=npy.concatenate((self._lbg, self._lbh)),
             ubg=0,
             lam_x0=0,
-            lam_g0=0
+            lam_g0=0,
         )
-        x = sol['x']
-        lam_g = sol['lam_g'][:self.ng, :]
-        lam_h = sol['lam_g'][self.ng:, :]
-        lam_lbx = -cs.fmin(sol['lam_x'], 0)
-        lam_ubx = cs.fmax(sol['lam_x'], 0)
-        Fsol = cs.Function('Fsol', ins, [x, lam_g, lam_h, lam_lbx, lam_ubx])
+        x = sol["x"]
+        lam_g = sol["lam_g"][: self.ng, :]
+        lam_h = sol["lam_g"][self.ng :, :]
+        lam_lbx = -cs.fmin(sol["lam_x"], 0)
+        lam_ubx = cs.fmax(sol["lam_x"], 0)
+        Fsol = cs.Function("Fsol", ins, [x, lam_g, lam_h, lam_lbx, lam_ubx])
 
         # build final function
         final_outs = Fout(p, *Fsol(*ins))
@@ -730,16 +742,18 @@ class Nlp:
         return cs.Function(*args)
 
     def __str__(self) -> str:
-        '''Returns the NLP name and a short description.'''
-        msg = 'not initialized' if self._solver is None else 'initialized'
+        """Returns the NLP name and a short description."""
+        msg = "not initialized" if self._solver is None else "initialized"
         C = len(self._cons)
-        return f'{type(self).__name__} {{\n' \
-               f'  name: {self.name}\n' \
-               f'  #variables: {len(self._vars)} (nx={self.nx})\n' \
-               f'  #parameters: {len(self._pars)} (np={self.np})\n' \
-               f'  #constraints: {C} (ng={self.ng}, nh={self.nh})\n' \
-               f'  CasADi solver {msg}.\n}}'
+        return (
+            f"{type(self).__name__} {{\n"
+            f"  name: {self.name}\n"
+            f"  #variables: {len(self._vars)} (nx={self.nx})\n"
+            f"  #parameters: {len(self._pars)} (np={self.np})\n"
+            f"  #constraints: {C} (ng={self.ng}, nh={self.nh})\n"
+            f"  CasADi solver {msg}.\n}}"
+        )
 
     def __repr__(self) -> str:
-        '''Returns the string representation of the NLP instance.'''
-        return f'{type(self).__name__}: {self.name}'
+        """Returns the string representation of the NLP instance."""
+        return f"{type(self).__name__}: {self.name}"
